# 内存管理模块设计与实现

* Author: iPear

## 涉及的代码文件

```
lib/memory.h(.c)
lib/kernel_stack.h(.c)
lib/kernel_heap.h(.c)
```

## 模块设计

内核的内存管理模块负责RAM的空间分配。分为内核堆管理和全局页面管理两部分。

K210开发板的整个内存约为6M，关闭KPU后可使用8M。通过汇编中嵌入标签__kernel_end，可以知道内核代码结束的位置，在此以后的空间都可以被分配，由全局页面管理模块控制。

初始化memory时，从__kernel_end到内存结束的空间被按照4K对齐，分为若干页面；随后初始化kernel_heap，该分配器向页面分配器申请若干页（编译时指定好）作为内核堆区，并以一个更小的单位进行对齐（一般为8或16bytes）。在申请内存时，不论是内核堆还是页面，都必须申请对应内存分配器所设定对齐单元大小（align_unit_size）的整数倍，这个对齐操作由分配器自动进行。

## 分配方案

两个内存分配器结构基本相同，为首先追求速度，都采用最先匹配的方案。数据结构上，各使用一颗线段树来实现。

![线段树](\assets\线段树.jpg)

分配器将内存对齐后，划分为若干需要管理的内存结点。如图，假设页面分配器共需管理41K内存，按4K对齐后有40K内存，被分为10个节点，记需要管理的内存范围为1-10，交给线段树的根节点管理。根节点将这个范围平分为1-5、6-10两部分，分给他的两个子节点管理；子节点也不断重复这一过程，直到节点管理的内存大小为1页为止。

对于每个节点，维护以下数据：

```
typedef struct {
    // left  node : [l,mid]
    // right node : [mid+1,r]
    int l, r;     // 该节点管理的内存左边界
    int mid;      // 该节点管理的内存右边界

    int left_boarder_space;     // 左子树管理的内存中，最左侧共有多少连续的空闲空间
    int left_right_space;       // 左子树管理的内存中，最右侧共有多少连续的空闲空间
    int right_left_space;       // 右子树管理的内存中，最左侧共有多少连续的空闲空间
    int right_boarder_space;    // 右子树管理的内存中，最右侧共有多少连续的空闲空间

    int left_max_space;         // 该节点管理的内存中，最左侧共有多少连续的空闲空间
    int right_max_space;        // 该节点管理的内存中，最右侧共有多少连续的空闲空间
    int max_space;              // 该节点管理的内存中，最大的连续空闲空间长度

    // 0:not set; 1:set to occupied; -1:set to unoccupied
    int set;                    // 该节点管理的内存是否被整体占用或释放
} __Memory_SegmentTreeNode;
```

分配内存时，由于已经缓存了子节点的信息，分配算法可以在log(N)的时间复杂度内得到首次匹配的地址。数据结构的空间复杂度为O(N)。